# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.realtime_kit import (
    meeting_get_params,
    meeting_create_params,
    meeting_add_participant_params,
    meeting_edit_participant_params,
    meeting_get_meeting_by_id_params,
    meeting_update_meeting_by_id_params,
    meeting_replace_meeting_by_id_params,
    meeting_get_meeting_participants_params,
)
from ...types.realtime_kit.meeting_get_response import MeetingGetResponse
from ...types.realtime_kit.meeting_create_response import MeetingCreateResponse
from ...types.realtime_kit.meeting_add_participant_response import MeetingAddParticipantResponse
from ...types.realtime_kit.meeting_edit_participant_response import MeetingEditParticipantResponse
from ...types.realtime_kit.meeting_get_meeting_by_id_response import MeetingGetMeetingByIDResponse
from ...types.realtime_kit.meeting_update_meeting_by_id_response import MeetingUpdateMeetingByIDResponse
from ...types.realtime_kit.meeting_replace_meeting_by_id_response import MeetingReplaceMeetingByIDResponse
from ...types.realtime_kit.meeting_get_meeting_participant_response import MeetingGetMeetingParticipantResponse
from ...types.realtime_kit.meeting_get_meeting_participants_response import MeetingGetMeetingParticipantsResponse
from ...types.realtime_kit.meeting_refresh_participant_token_response import MeetingRefreshParticipantTokenResponse
from ...types.realtime_kit.meeting_delete_meeting_participant_response import MeetingDeleteMeetingParticipantResponse

__all__ = ["MeetingsResource", "AsyncMeetingsResource"]


class MeetingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MeetingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MeetingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MeetingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MeetingsResourceWithStreamingResponse(self)

    def create(
        self,
        app_id: str,
        *,
        account_id: str,
        ai_config: meeting_create_params.AIConfig | Omit = omit,
        live_stream_on_start: Optional[bool] | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: Optional[bool] | Omit = omit,
        recording_config: meeting_create_params.RecordingConfig | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingCreateResponse:
        """
        Create a meeting for the given App ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is set to persist_chat, meeting chat would remain for a week within
              the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          recording_config: Recording Configurations to be used for this meeting. This level of configs
              takes higher preference over App level configs on the RealtimeKit developer
              portal.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings", account_id=account_id, app_id=app_id
            ),
            body=maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "recording_config": recording_config,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_create_params.MeetingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingCreateResponse,
        )

    def add_participant(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        custom_participant_id: str,
        preset_name: str,
        name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingAddParticipantResponse:
        """
        Adds a participant to the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          custom_participant_id: A unique participant ID. You must specify a unique ID for the participant, for
              example, UUID, email address, and so on.

          preset_name: Name of the preset to apply to this participant.

          name: (Optional) Name of the participant.

          picture: (Optional) A URL to a picture to be used for the participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=maybe_transform(
                {
                    "custom_participant_id": custom_participant_id,
                    "preset_name": preset_name,
                    "name": name,
                    "picture": picture,
                },
                meeting_add_participant_params.MeetingAddParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingAddParticipantResponse,
        )

    def delete_meeting_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingDeleteMeetingParticipantResponse:
        """
        Deletes a participant for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingDeleteMeetingParticipantResponse,
        )

    def edit_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        preset_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingEditParticipantResponse:
        """
        Updates a participant's details for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: (Optional) Name of the participant.

          picture: (Optional) A URL to a picture to be used for the participant.

          preset_name: (Optional) Name of the preset to apply to this participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "picture": picture,
                    "preset_name": preset_name,
                },
                meeting_edit_participant_params.MeetingEditParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingEditParticipantResponse,
        )

    def get(
        self,
        app_id: str,
        *,
        account_id: str,
        end_time: Union[str, datetime] | Omit = omit,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetResponse:
        """
        Returns all meetings for the given App ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          end_time: The end time range for which you want to retrieve the meetings. The time must be
              specified in ISO format.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          search: The search query string. You can search using the meeting ID or title.

          start_time: The start time range for which you want to retrieve the meetings. The time must
              be specified in ISO format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_time": end_time,
                        "page_no": page_no,
                        "per_page": per_page,
                        "search": search,
                        "start_time": start_time,
                    },
                    meeting_get_params.MeetingGetParams,
                ),
            ),
            cast_to=MeetingGetResponse,
        )

    def get_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingByIDResponse:
        """
        Returns a meeting details in an App for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"name": name}, meeting_get_meeting_by_id_params.MeetingGetMeetingByIDParams),
            ),
            cast_to=MeetingGetMeetingByIDResponse,
        )

    def get_meeting_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingParticipantResponse:
        """
        Returns a participant details for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingGetMeetingParticipantResponse,
        )

    def get_meeting_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingParticipantsResponse:
        """
        Returns all participants detail for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_no": page_no,
                        "per_page": per_page,
                    },
                    meeting_get_meeting_participants_params.MeetingGetMeetingParticipantsParams,
                ),
            ),
            cast_to=MeetingGetMeetingParticipantsResponse,
        )

    def refresh_participant_token(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingRefreshParticipantTokenResponse:
        """
        Regenerates participant's authentication token for the given meeting and
        participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}/token",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingRefreshParticipantTokenResponse,
        )

    def replace_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        ai_config: meeting_replace_meeting_by_id_params.AIConfig | Omit = omit,
        live_stream_on_start: Optional[bool] | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: Optional[bool] | Omit = omit,
        recording_config: meeting_replace_meeting_by_id_params.RecordingConfig | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingReplaceMeetingByIDResponse:
        """
        Replaces all the details for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is set to persist_chat, meeting chat would remain for a week within
              the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          recording_config: Recording Configurations to be used for this meeting. This level of configs
              takes higher preference over App level configs on the RealtimeKit developer
              portal.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "recording_config": recording_config,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_replace_meeting_by_id_params.MeetingReplaceMeetingByIDParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingReplaceMeetingByIDResponse,
        )

    def update_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        ai_config: meeting_update_meeting_by_id_params.AIConfig | Omit = omit,
        live_stream_on_start: bool | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: bool | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        status: Literal["ACTIVE", "INACTIVE"] | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingUpdateMeetingByIDResponse:
        """
        Updates a meeting in an App for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is updated to persist_chat, meeting chat would remain for a week
              within the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          status: Whether the meeting is `ACTIVE` or `INACTIVE`. Users will not be able to join an
              `INACTIVE` meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "status": status,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_update_meeting_by_id_params.MeetingUpdateMeetingByIDParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingUpdateMeetingByIDResponse,
        )


class AsyncMeetingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMeetingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMeetingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMeetingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMeetingsResourceWithStreamingResponse(self)

    async def create(
        self,
        app_id: str,
        *,
        account_id: str,
        ai_config: meeting_create_params.AIConfig | Omit = omit,
        live_stream_on_start: Optional[bool] | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: Optional[bool] | Omit = omit,
        recording_config: meeting_create_params.RecordingConfig | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingCreateResponse:
        """
        Create a meeting for the given App ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is set to persist_chat, meeting chat would remain for a week within
              the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          recording_config: Recording Configurations to be used for this meeting. This level of configs
              takes higher preference over App level configs on the RealtimeKit developer
              portal.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings", account_id=account_id, app_id=app_id
            ),
            body=await async_maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "recording_config": recording_config,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_create_params.MeetingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingCreateResponse,
        )

    async def add_participant(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        custom_participant_id: str,
        preset_name: str,
        name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingAddParticipantResponse:
        """
        Adds a participant to the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          custom_participant_id: A unique participant ID. You must specify a unique ID for the participant, for
              example, UUID, email address, and so on.

          preset_name: Name of the preset to apply to this participant.

          name: (Optional) Name of the participant.

          picture: (Optional) A URL to a picture to be used for the participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=await async_maybe_transform(
                {
                    "custom_participant_id": custom_participant_id,
                    "preset_name": preset_name,
                    "name": name,
                    "picture": picture,
                },
                meeting_add_participant_params.MeetingAddParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingAddParticipantResponse,
        )

    async def delete_meeting_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingDeleteMeetingParticipantResponse:
        """
        Deletes a participant for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingDeleteMeetingParticipantResponse,
        )

    async def edit_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        name: Optional[str] | Omit = omit,
        picture: Optional[str] | Omit = omit,
        preset_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingEditParticipantResponse:
        """
        Updates a participant's details for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          name: (Optional) Name of the participant.

          picture: (Optional) A URL to a picture to be used for the participant.

          preset_name: (Optional) Name of the preset to apply to this participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "picture": picture,
                    "preset_name": preset_name,
                },
                meeting_edit_participant_params.MeetingEditParticipantParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingEditParticipantResponse,
        )

    async def get(
        self,
        app_id: str,
        *,
        account_id: str,
        end_time: Union[str, datetime] | Omit = omit,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetResponse:
        """
        Returns all meetings for the given App ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          end_time: The end time range for which you want to retrieve the meetings. The time must be
              specified in ISO format.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          search: The search query string. You can search using the meeting ID or title.

          start_time: The start time range for which you want to retrieve the meetings. The time must
              be specified in ISO format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_time": end_time,
                        "page_no": page_no,
                        "per_page": per_page,
                        "search": search,
                        "start_time": start_time,
                    },
                    meeting_get_params.MeetingGetParams,
                ),
            ),
            cast_to=MeetingGetResponse,
        )

    async def get_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingByIDResponse:
        """
        Returns a meeting details in an App for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"name": name}, meeting_get_meeting_by_id_params.MeetingGetMeetingByIDParams
                ),
            ),
            cast_to=MeetingGetMeetingByIDResponse,
        )

    async def get_meeting_participant(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingParticipantResponse:
        """
        Returns a participant details for the given meeting and participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingGetMeetingParticipantResponse,
        )

    async def get_meeting_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingGetMeetingParticipantsResponse:
        """
        Returns all participants detail for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_no": page_no,
                        "per_page": per_page,
                    },
                    meeting_get_meeting_participants_params.MeetingGetMeetingParticipantsParams,
                ),
            ),
            cast_to=MeetingGetMeetingParticipantsResponse,
        )

    async def refresh_participant_token(
        self,
        participant_id: str,
        *,
        account_id: str,
        app_id: str,
        meeting_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingRefreshParticipantTokenResponse:
        """
        Regenerates participant's authentication token for the given meeting and
        participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        if not participant_id:
            raise ValueError(f"Expected a non-empty value for `participant_id` but received {participant_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/participants/{participant_id}/token",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
                participant_id=participant_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingRefreshParticipantTokenResponse,
        )

    async def replace_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        ai_config: meeting_replace_meeting_by_id_params.AIConfig | Omit = omit,
        live_stream_on_start: Optional[bool] | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: Optional[bool] | Omit = omit,
        recording_config: meeting_replace_meeting_by_id_params.RecordingConfig | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingReplaceMeetingByIDResponse:
        """
        Replaces all the details for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is set to persist_chat, meeting chat would remain for a week within
              the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          recording_config: Recording Configurations to be used for this meeting. This level of configs
              takes higher preference over App level configs on the RealtimeKit developer
              portal.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=await async_maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "recording_config": recording_config,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_replace_meeting_by_id_params.MeetingReplaceMeetingByIDParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingReplaceMeetingByIDResponse,
        )

    async def update_meeting_by_id(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        ai_config: meeting_update_meeting_by_id_params.AIConfig | Omit = omit,
        live_stream_on_start: bool | Omit = omit,
        persist_chat: bool | Omit = omit,
        record_on_start: bool | Omit = omit,
        session_keep_alive_time_in_secs: float | Omit = omit,
        status: Literal["ACTIVE", "INACTIVE"] | Omit = omit,
        summarize_on_end: bool | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeetingUpdateMeetingByIDResponse:
        """
        Updates a meeting in an App for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          ai_config: The AI Config allows you to customize the behavior of meeting transcriptions and
              summaries

          live_stream_on_start: Specifies if the meeting should start getting livestreamed on start.

          persist_chat: If a meeting is updated to persist_chat, meeting chat would remain for a week
              within the meeting space.

          record_on_start: Specifies if the meeting should start getting recorded as soon as someone joins
              the meeting.

          session_keep_alive_time_in_secs: Time in seconds, for which a session remains active, after the last participant
              has left the meeting.

          status: Whether the meeting is `ACTIVE` or `INACTIVE`. Users will not be able to join an
              `INACTIVE` meeting.

          summarize_on_end: Automatically generate summary of meetings using transcripts. Requires
              Transcriptions to be enabled, and can be retrieved via Webhooks or summary API.

          title: Title of the meeting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=await async_maybe_transform(
                {
                    "ai_config": ai_config,
                    "live_stream_on_start": live_stream_on_start,
                    "persist_chat": persist_chat,
                    "record_on_start": record_on_start,
                    "session_keep_alive_time_in_secs": session_keep_alive_time_in_secs,
                    "status": status,
                    "summarize_on_end": summarize_on_end,
                    "title": title,
                },
                meeting_update_meeting_by_id_params.MeetingUpdateMeetingByIDParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeetingUpdateMeetingByIDResponse,
        )


class MeetingsResourceWithRawResponse:
    def __init__(self, meetings: MeetingsResource) -> None:
        self._meetings = meetings

        self.create = to_raw_response_wrapper(
            meetings.create,
        )
        self.add_participant = to_raw_response_wrapper(
            meetings.add_participant,
        )
        self.delete_meeting_participant = to_raw_response_wrapper(
            meetings.delete_meeting_participant,
        )
        self.edit_participant = to_raw_response_wrapper(
            meetings.edit_participant,
        )
        self.get = to_raw_response_wrapper(
            meetings.get,
        )
        self.get_meeting_by_id = to_raw_response_wrapper(
            meetings.get_meeting_by_id,
        )
        self.get_meeting_participant = to_raw_response_wrapper(
            meetings.get_meeting_participant,
        )
        self.get_meeting_participants = to_raw_response_wrapper(
            meetings.get_meeting_participants,
        )
        self.refresh_participant_token = to_raw_response_wrapper(
            meetings.refresh_participant_token,
        )
        self.replace_meeting_by_id = to_raw_response_wrapper(
            meetings.replace_meeting_by_id,
        )
        self.update_meeting_by_id = to_raw_response_wrapper(
            meetings.update_meeting_by_id,
        )


class AsyncMeetingsResourceWithRawResponse:
    def __init__(self, meetings: AsyncMeetingsResource) -> None:
        self._meetings = meetings

        self.create = async_to_raw_response_wrapper(
            meetings.create,
        )
        self.add_participant = async_to_raw_response_wrapper(
            meetings.add_participant,
        )
        self.delete_meeting_participant = async_to_raw_response_wrapper(
            meetings.delete_meeting_participant,
        )
        self.edit_participant = async_to_raw_response_wrapper(
            meetings.edit_participant,
        )
        self.get = async_to_raw_response_wrapper(
            meetings.get,
        )
        self.get_meeting_by_id = async_to_raw_response_wrapper(
            meetings.get_meeting_by_id,
        )
        self.get_meeting_participant = async_to_raw_response_wrapper(
            meetings.get_meeting_participant,
        )
        self.get_meeting_participants = async_to_raw_response_wrapper(
            meetings.get_meeting_participants,
        )
        self.refresh_participant_token = async_to_raw_response_wrapper(
            meetings.refresh_participant_token,
        )
        self.replace_meeting_by_id = async_to_raw_response_wrapper(
            meetings.replace_meeting_by_id,
        )
        self.update_meeting_by_id = async_to_raw_response_wrapper(
            meetings.update_meeting_by_id,
        )


class MeetingsResourceWithStreamingResponse:
    def __init__(self, meetings: MeetingsResource) -> None:
        self._meetings = meetings

        self.create = to_streamed_response_wrapper(
            meetings.create,
        )
        self.add_participant = to_streamed_response_wrapper(
            meetings.add_participant,
        )
        self.delete_meeting_participant = to_streamed_response_wrapper(
            meetings.delete_meeting_participant,
        )
        self.edit_participant = to_streamed_response_wrapper(
            meetings.edit_participant,
        )
        self.get = to_streamed_response_wrapper(
            meetings.get,
        )
        self.get_meeting_by_id = to_streamed_response_wrapper(
            meetings.get_meeting_by_id,
        )
        self.get_meeting_participant = to_streamed_response_wrapper(
            meetings.get_meeting_participant,
        )
        self.get_meeting_participants = to_streamed_response_wrapper(
            meetings.get_meeting_participants,
        )
        self.refresh_participant_token = to_streamed_response_wrapper(
            meetings.refresh_participant_token,
        )
        self.replace_meeting_by_id = to_streamed_response_wrapper(
            meetings.replace_meeting_by_id,
        )
        self.update_meeting_by_id = to_streamed_response_wrapper(
            meetings.update_meeting_by_id,
        )


class AsyncMeetingsResourceWithStreamingResponse:
    def __init__(self, meetings: AsyncMeetingsResource) -> None:
        self._meetings = meetings

        self.create = async_to_streamed_response_wrapper(
            meetings.create,
        )
        self.add_participant = async_to_streamed_response_wrapper(
            meetings.add_participant,
        )
        self.delete_meeting_participant = async_to_streamed_response_wrapper(
            meetings.delete_meeting_participant,
        )
        self.edit_participant = async_to_streamed_response_wrapper(
            meetings.edit_participant,
        )
        self.get = async_to_streamed_response_wrapper(
            meetings.get,
        )
        self.get_meeting_by_id = async_to_streamed_response_wrapper(
            meetings.get_meeting_by_id,
        )
        self.get_meeting_participant = async_to_streamed_response_wrapper(
            meetings.get_meeting_participant,
        )
        self.get_meeting_participants = async_to_streamed_response_wrapper(
            meetings.get_meeting_participants,
        )
        self.refresh_participant_token = async_to_streamed_response_wrapper(
            meetings.refresh_participant_token,
        )
        self.replace_meeting_by_id = async_to_streamed_response_wrapper(
            meetings.replace_meeting_by_id,
        )
        self.update_meeting_by_id = async_to_streamed_response_wrapper(
            meetings.update_meeting_by_id,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["DeviceGetResponse", "LastSeenRegistration", "LastSeenRegistrationPolicy", "LastSeenUser"]


class LastSeenRegistrationPolicy(BaseModel):
    """A summary of the device profile evaluated for the registration."""

    id: str
    """The ID of the device settings profile."""

    default: bool
    """Whether the device settings profile is the default profile for the account."""

    deleted: bool
    """Whether the device settings profile was deleted."""

    name: str
    """The name of the device settings profile."""

    updated_at: str
    """
    The RFC3339 timestamp of when the device settings profile last changed for the
    registration.
    """


class LastSeenRegistration(BaseModel):
    """The last seen registration for the device."""

    policy: Optional[LastSeenRegistrationPolicy] = None
    """A summary of the device profile evaluated for the registration."""


class LastSeenUser(BaseModel):
    """The last user to use the WARP device."""

    id: Optional[str] = None
    """UUID."""

    email: Optional[str] = None
    """The contact email address of the user."""

    name: Optional[str] = None
    """The enrolled device user's name."""


class DeviceGetResponse(BaseModel):
    """A WARP Device."""

    id: str
    """The unique ID of the device."""

    active_registrations: int
    """The number of active registrations for the device.

    Active registrations are those which haven't been revoked or deleted.
    """

    created_at: str
    """The RFC3339 timestamp when the device was created."""

    last_seen_at: Optional[str] = None
    """The RFC3339 timestamp when the device was last seen."""

    name: str
    """The name of the device."""

    updated_at: str
    """The RFC3339 timestamp when the device was last updated."""

    client_version: Optional[str] = None
    """Version of the WARP client."""

    deleted_at: Optional[str] = None
    """The RFC3339 timestamp when the device was deleted."""

    device_type: Optional[str] = None
    """The device operating system."""

    hardware_id: Optional[str] = None
    """A string that uniquely identifies the hardware or virtual machine (VM)."""

    last_seen_registration: Optional[LastSeenRegistration] = None
    """The last seen registration for the device."""

    last_seen_user: Optional[LastSeenUser] = None
    """The last user to use the WARP device."""

    mac_address: Optional[str] = None
    """The device MAC address."""

    manufacturer: Optional[str] = None
    """The device manufacturer."""

    model: Optional[str] = None
    """The model name of the device."""

    os_version: Optional[str] = None
    """The device operating system version number."""

    os_version_extra: Optional[str] = None
    """Additional operating system version details.

    For Windows, the UBR (Update Build Revision). For Mac or iOS, the Product
    Version Extra. For Linux, the distribution name and version.
    """

    public_ip: Optional[str] = None
    """
    **Deprecated**: IP information is provided by DEX - see
    https://developers.cloudflare.com/api/resources/zero_trust/subresources/dex/subresources/fleet_status/subresources/devices/methods/list/
    """

    serial_number: Optional[str] = None
    """The device serial number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["RouteRealtimeResponse", "Meta", "MetaASNInfo", "MetaCollector", "MetaPrefixOrigin", "Route"]


class MetaASNInfo(BaseModel):
    as_name: str
    """Name of the autonomous system."""

    asn: int
    """AS number."""

    country_code: str
    """Alpha-2 code for the AS's registration country."""

    org_id: str
    """Organization ID."""

    org_name: str
    """Organization name."""


class MetaCollector(BaseModel):
    collector: str
    """Public route collector ID."""

    latest_realtime_ts: str
    """Latest real-time stream timestamp for this collector."""

    latest_rib_ts: str
    """Latest RIB dump MRT file timestamp for this collector."""

    latest_updates_ts: str
    """Latest BGP updates MRT file timestamp for this collector."""

    peers_count: int
    """Total number of collector peers used from this collector."""

    peers_v4_count: int
    """Total number of collector peers used from this collector for IPv4 prefixes."""

    peers_v6_count: int
    """Total number of collector peers used from this collector for IPv6 prefixes."""


class MetaPrefixOrigin(BaseModel):
    origin: int
    """Origin ASN."""

    prefix: str
    """IP prefix of this query."""

    rpki_validation: str
    """Prefix-origin RPKI validation: valid, invalid, unknown."""

    total_peers: int
    """Total number of peers."""

    total_visible: int
    """Total number of peers seeing this prefix."""

    visibility: float
    """Ratio of peers seeing this prefix to total number of peers."""


class Meta(BaseModel):
    asn_info: List[MetaASNInfo]

    collectors: List[MetaCollector]

    data_time: str
    """The most recent data timestamp for from the real-time sources."""

    prefix_origins: List[MetaPrefixOrigin]

    query_time: str
    """The timestamp of this query."""


class Route(BaseModel):
    as_path: List[int]
    """AS-level path for this route, from collector to origin."""

    collector: str
    """Public collector ID for this route."""

    communities: List[str]
    """BGP community values."""

    prefix: str
    """IP prefix of this query."""

    timestamp: str
    """Latest timestamp of change for this route."""


class RouteRealtimeResponse(BaseModel):
    meta: Meta

    routes: List[Route]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.email_routing.rules import catch_all_update_params
from ....types.email_routing.rules.catch_all_action_param import CatchAllActionParam
from ....types.email_routing.rules.catch_all_get_response import CatchAllGetResponse
from ....types.email_routing.rules.catch_all_matcher_param import CatchAllMatcherParam
from ....types.email_routing.rules.catch_all_update_response import CatchAllUpdateResponse

__all__ = ["CatchAllsResource", "AsyncCatchAllsResource"]


class CatchAllsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CatchAllsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CatchAllsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CatchAllsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CatchAllsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        actions: Iterable[CatchAllActionParam],
        matchers: Iterable[CatchAllMatcherParam],
        enabled: Literal[True, False] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CatchAllUpdateResponse]:
        """
        Enable or disable catch-all routing rule, or change action to forward to
        specific destination address. Forward actions require all destination addresses
        to be verified.

        Args:
          zone_id: Identifier.

          actions: List actions for the catch-all routing rule.

          matchers: List of matchers for the catch-all routing rule.

          enabled: Routing rule status.

          name: Routing rule name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/email/routing/rules/catch_all", zone_id=zone_id),
            body=maybe_transform(
                {
                    "actions": actions,
                    "matchers": matchers,
                    "enabled": enabled,
                    "name": name,
                },
                catch_all_update_params.CatchAllUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CatchAllUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CatchAllUpdateResponse]], ResultWrapper[CatchAllUpdateResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CatchAllGetResponse]:
        """
        Get information on the default catch-all routing rule.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/email/routing/rules/catch_all", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CatchAllGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CatchAllGetResponse]], ResultWrapper[CatchAllGetResponse]),
        )


class AsyncCatchAllsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCatchAllsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCatchAllsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCatchAllsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCatchAllsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        zone_id: str,
        actions: Iterable[CatchAllActionParam],
        matchers: Iterable[CatchAllMatcherParam],
        enabled: Literal[True, False] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CatchAllUpdateResponse]:
        """
        Enable or disable catch-all routing rule, or change action to forward to
        specific destination address. Forward actions require all destination addresses
        to be verified.

        Args:
          zone_id: Identifier.

          actions: List actions for the catch-all routing rule.

          matchers: List of matchers for the catch-all routing rule.

          enabled: Routing rule status.

          name: Routing rule name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/email/routing/rules/catch_all", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "actions": actions,
                    "matchers": matchers,
                    "enabled": enabled,
                    "name": name,
                },
                catch_all_update_params.CatchAllUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CatchAllUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CatchAllUpdateResponse]], ResultWrapper[CatchAllUpdateResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CatchAllGetResponse]:
        """
        Get information on the default catch-all routing rule.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/email/routing/rules/catch_all", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CatchAllGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CatchAllGetResponse]], ResultWrapper[CatchAllGetResponse]),
        )


class CatchAllsResourceWithRawResponse:
    def __init__(self, catch_alls: CatchAllsResource) -> None:
        self._catch_alls = catch_alls

        self.update = to_raw_response_wrapper(
            catch_alls.update,
        )
        self.get = to_raw_response_wrapper(
            catch_alls.get,
        )


class AsyncCatchAllsResourceWithRawResponse:
    def __init__(self, catch_alls: AsyncCatchAllsResource) -> None:
        self._catch_alls = catch_alls

        self.update = async_to_raw_response_wrapper(
            catch_alls.update,
        )
        self.get = async_to_raw_response_wrapper(
            catch_alls.get,
        )


class CatchAllsResourceWithStreamingResponse:
    def __init__(self, catch_alls: CatchAllsResource) -> None:
        self._catch_alls = catch_alls

        self.update = to_streamed_response_wrapper(
            catch_alls.update,
        )
        self.get = to_streamed_response_wrapper(
            catch_alls.get,
        )


class AsyncCatchAllsResourceWithStreamingResponse:
    def __init__(self, catch_alls: AsyncCatchAllsResource) -> None:
        self._catch_alls = catch_alls

        self.update = async_to_streamed_response_wrapper(
            catch_alls.update,
        )
        self.get = async_to_streamed_response_wrapper(
            catch_alls.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .smart_routing import (
    SmartRoutingResource,
    AsyncSmartRoutingResource,
    SmartRoutingResourceWithRawResponse,
    AsyncSmartRoutingResourceWithRawResponse,
    SmartRoutingResourceWithStreamingResponse,
    AsyncSmartRoutingResourceWithStreamingResponse,
)
from .tiered_caching import (
    TieredCachingResource,
    AsyncTieredCachingResource,
    TieredCachingResourceWithRawResponse,
    AsyncTieredCachingResourceWithRawResponse,
    TieredCachingResourceWithStreamingResponse,
    AsyncTieredCachingResourceWithStreamingResponse,
)

__all__ = ["ArgoResource", "AsyncArgoResource"]


class ArgoResource(SyncAPIResource):
    @cached_property
    def smart_routing(self) -> SmartRoutingResource:
        return SmartRoutingResource(self._client)

    @cached_property
    def tiered_caching(self) -> TieredCachingResource:
        return TieredCachingResource(self._client)

    @cached_property
    def with_raw_response(self) -> ArgoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ArgoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArgoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ArgoResourceWithStreamingResponse(self)


class AsyncArgoResource(AsyncAPIResource):
    @cached_property
    def smart_routing(self) -> AsyncSmartRoutingResource:
        return AsyncSmartRoutingResource(self._client)

    @cached_property
    def tiered_caching(self) -> AsyncTieredCachingResource:
        return AsyncTieredCachingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncArgoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncArgoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArgoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncArgoResourceWithStreamingResponse(self)


class ArgoResourceWithRawResponse:
    def __init__(self, argo: ArgoResource) -> None:
        self._argo = argo

    @cached_property
    def smart_routing(self) -> SmartRoutingResourceWithRawResponse:
        return SmartRoutingResourceWithRawResponse(self._argo.smart_routing)

    @cached_property
    def tiered_caching(self) -> TieredCachingResourceWithRawResponse:
        return TieredCachingResourceWithRawResponse(self._argo.tiered_caching)


class AsyncArgoResourceWithRawResponse:
    def __init__(self, argo: AsyncArgoResource) -> None:
        self._argo = argo

    @cached_property
    def smart_routing(self) -> AsyncSmartRoutingResourceWithRawResponse:
        return AsyncSmartRoutingResourceWithRawResponse(self._argo.smart_routing)

    @cached_property
    def tiered_caching(self) -> AsyncTieredCachingResourceWithRawResponse:
        return AsyncTieredCachingResourceWithRawResponse(self._argo.tiered_caching)


class ArgoResourceWithStreamingResponse:
    def __init__(self, argo: ArgoResource) -> None:
        self._argo = argo

    @cached_property
    def smart_routing(self) -> SmartRoutingResourceWithStreamingResponse:
        return SmartRoutingResourceWithStreamingResponse(self._argo.smart_routing)

    @cached_property
    def tiered_caching(self) -> TieredCachingResourceWithStreamingResponse:
        return TieredCachingResourceWithStreamingResponse(self._argo.tiered_caching)


class AsyncArgoResourceWithStreamingResponse:
    def __init__(self, argo: AsyncArgoResource) -> None:
        self._argo = argo

    @cached_property
    def smart_routing(self) -> AsyncSmartRoutingResourceWithStreamingResponse:
        return AsyncSmartRoutingResourceWithStreamingResponse(self._argo.smart_routing)

    @cached_property
    def tiered_caching(self) -> AsyncTieredCachingResourceWithStreamingResponse:
        return AsyncTieredCachingResourceWithStreamingResponse(self._argo.tiered_caching)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["DomainGetResponse", "Details0", "Details0Category", "Details0TopLocation", "Meta", "MetaDateRange"]


class Details0Category(BaseModel):
    id: int

    name: str

    super_category_id: int = FieldInfo(alias="superCategoryId")


class Details0TopLocation(BaseModel):
    location_code: str = FieldInfo(alias="locationCode")

    location_name: str = FieldInfo(alias="locationName")

    rank: int


class Details0(BaseModel):
    categories: List[Details0Category]

    bucket: Optional[str] = None
    """Only available in POPULAR ranking for the most recent ranking."""

    rank: Optional[int] = None

    top_locations: Optional[List[Details0TopLocation]] = None


class MetaDateRange(BaseModel):
    end_time: datetime = FieldInfo(alias="endTime")
    """Adjusted end of date range."""

    start_time: datetime = FieldInfo(alias="startTime")
    """Adjusted start of date range."""


class Meta(BaseModel):
    date_range: List[MetaDateRange] = FieldInfo(alias="dateRange")


class DomainGetResponse(BaseModel):
    details_0: Details0

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from .top.top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .timeseries_groups import (
    TimeseriesGroupsResource,
    AsyncTimeseriesGroupsResource,
    TimeseriesGroupsResourceWithRawResponse,
    AsyncTimeseriesGroupsResourceWithRawResponse,
    TimeseriesGroupsResourceWithStreamingResponse,
    AsyncTimeseriesGroupsResourceWithStreamingResponse,
)
from .....types.radar.email import security_summary_v2_params, security_timeseries_groups_v2_params
from .....types.radar.email.security_summary_v2_response import SecuritySummaryV2Response
from .....types.radar.email.security_timeseries_groups_v2_response import SecurityTimeseriesGroupsV2Response

__all__ = ["SecurityResource", "AsyncSecurityResource"]


class SecurityResource(SyncAPIResource):
    @cached_property
    def top(self) -> TopResource:
        return TopResource(self._client)

    @cached_property
    def summary(self) -> SummaryResource:
        return SummaryResource(self._client)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResource:
        return TimeseriesGroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SecurityResourceWithStreamingResponse(self)

    def summary_v2(
        self,
        dimension: Literal[
            "SPAM", "MALICIOUS", "SPOOF", "THREAT_CATEGORY", "ARC", "DKIM", "DMARC", "SPF", "TLS_VERSION"
        ],
        *,
        arc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        dkim: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        dmarc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        spf: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        tls_version: List[Literal["TLSv1_0", "TLSv1_1", "TLSv1_2", "TLSv1_3"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SecuritySummaryV2Response:
        """
        Retrieves the distribution of email security metrics by the specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          arc: Filters results by ARC (Authenticated Received Chain) validation.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          dkim: Filters results by DKIM (DomainKeys Identified Mail) validation status.

          dmarc: Filters results by DMARC (Domain-based Message Authentication, Reporting and
              Conformance) validation status.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          spf: Filters results by SPF (Sender Policy Framework) validation status.

          tls_version: Filters results by TLS version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/email/security/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "arc": arc,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "dkim": dkim,
                        "dmarc": dmarc,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "spf": spf,
                        "tls_version": tls_version,
                    },
                    security_summary_v2_params.SecuritySummaryV2Params,
                ),
                post_parser=ResultWrapper[SecuritySummaryV2Response]._unwrapper,
            ),
            cast_to=cast(Type[SecuritySummaryV2Response], ResultWrapper[SecuritySummaryV2Response]),
        )

    def timeseries_groups_v2(
        self,
        dimension: Literal[
            "SPAM", "MALICIOUS", "SPOOF", "THREAT_CATEGORY", "ARC", "DKIM", "DMARC", "SPF", "TLS_VERSION"
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        arc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        dkim: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        dmarc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        spf: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        tls_version: List[Literal["TLSv1_0", "TLSv1_1", "TLSv1_2", "TLSv1_3"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SecurityTimeseriesGroupsV2Response:
        """
        Retrieves the distribution of email security metrics grouped by dimension over
        time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          arc: Filters results by ARC (Authenticated Received Chain) validation.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          dkim: Filters results by DKIM (DomainKeys Identified Mail) validation status.

          dmarc: Filters results by DMARC (Domain-based Message Authentication, Reporting and
              Conformance) validation status.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          spf: Filters results by SPF (Sender Policy Framework) validation status.

          tls_version: Filters results by TLS version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/email/security/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "arc": arc,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "dkim": dkim,
                        "dmarc": dmarc,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "spf": spf,
                        "tls_version": tls_version,
                    },
                    security_timeseries_groups_v2_params.SecurityTimeseriesGroupsV2Params,
                ),
                post_parser=ResultWrapper[SecurityTimeseriesGroupsV2Response]._unwrapper,
            ),
            cast_to=cast(Type[SecurityTimeseriesGroupsV2Response], ResultWrapper[SecurityTimeseriesGroupsV2Response]),
        )


class AsyncSecurityResource(AsyncAPIResource):
    @cached_property
    def top(self) -> AsyncTopResource:
        return AsyncTopResource(self._client)

    @cached_property
    def summary(self) -> AsyncSummaryResource:
        return AsyncSummaryResource(self._client)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResource:
        return AsyncTimeseriesGroupsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSecurityResourceWithStreamingResponse(self)

    async def summary_v2(
        self,
        dimension: Literal[
            "SPAM", "MALICIOUS", "SPOOF", "THREAT_CATEGORY", "ARC", "DKIM", "DMARC", "SPF", "TLS_VERSION"
        ],
        *,
        arc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        dkim: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        dmarc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        spf: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        tls_version: List[Literal["TLSv1_0", "TLSv1_1", "TLSv1_2", "TLSv1_3"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SecuritySummaryV2Response:
        """
        Retrieves the distribution of email security metrics by the specified dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          arc: Filters results by ARC (Authenticated Received Chain) validation.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          dkim: Filters results by DKIM (DomainKeys Identified Mail) validation status.

          dmarc: Filters results by DMARC (Domain-based Message Authentication, Reporting and
              Conformance) validation status.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          spf: Filters results by SPF (Sender Policy Framework) validation status.

          tls_version: Filters results by TLS version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/email/security/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "arc": arc,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "dkim": dkim,
                        "dmarc": dmarc,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "spf": spf,
                        "tls_version": tls_version,
                    },
                    security_summary_v2_params.SecuritySummaryV2Params,
                ),
                post_parser=ResultWrapper[SecuritySummaryV2Response]._unwrapper,
            ),
            cast_to=cast(Type[SecuritySummaryV2Response], ResultWrapper[SecuritySummaryV2Response]),
        )

    async def timeseries_groups_v2(
        self,
        dimension: Literal[
            "SPAM", "MALICIOUS", "SPOOF", "THREAT_CATEGORY", "ARC", "DKIM", "DMARC", "SPF", "TLS_VERSION"
        ],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        arc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        dkim: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        dmarc: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        spf: List[Literal["PASS", "NONE", "FAIL"]] | Omit = omit,
        tls_version: List[Literal["TLSv1_0", "TLSv1_1", "TLSv1_2", "TLSv1_3"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SecurityTimeseriesGroupsV2Response:
        """
        Retrieves the distribution of email security metrics grouped by dimension over
        time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          arc: Filters results by ARC (Authenticated Received Chain) validation.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          dkim: Filters results by DKIM (DomainKeys Identified Mail) validation status.

          dmarc: Filters results by DMARC (Domain-based Message Authentication, Reporting and
              Conformance) validation status.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          name: Array of names used to label the series in the response.

          spf: Filters results by SPF (Sender Policy Framework) validation status.

          tls_version: Filters results by TLS version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/email/security/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "arc": arc,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "dkim": dkim,
                        "dmarc": dmarc,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "name": name,
                        "spf": spf,
                        "tls_version": tls_version,
                    },
                    security_timeseries_groups_v2_params.SecurityTimeseriesGroupsV2Params,
                ),
                post_parser=ResultWrapper[SecurityTimeseriesGroupsV2Response]._unwrapper,
            ),
            cast_to=cast(Type[SecurityTimeseriesGroupsV2Response], ResultWrapper[SecurityTimeseriesGroupsV2Response]),
        )


class SecurityResourceWithRawResponse:
    def __init__(self, security: SecurityResource) -> None:
        self._security = security

        self.summary_v2 = to_raw_response_wrapper(
            security.summary_v2,
        )
        self.timeseries_groups_v2 = to_raw_response_wrapper(
            security.timeseries_groups_v2,
        )

    @cached_property
    def top(self) -> TopResourceWithRawResponse:
        return TopResourceWithRawResponse(self._security.top)

    @cached_property
    def summary(self) -> SummaryResourceWithRawResponse:
        return SummaryResourceWithRawResponse(self._security.summary)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResourceWithRawResponse:
        return TimeseriesGroupsResourceWithRawResponse(self._security.timeseries_groups)


class AsyncSecurityResourceWithRawResponse:
    def __init__(self, security: AsyncSecurityResource) -> None:
        self._security = security

        self.summary_v2 = async_to_raw_response_wrapper(
            security.summary_v2,
        )
        self.timeseries_groups_v2 = async_to_raw_response_wrapper(
            security.timeseries_groups_v2,
        )

    @cached_property
    def top(self) -> AsyncTopResourceWithRawResponse:
        return AsyncTopResourceWithRawResponse(self._security.top)

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithRawResponse:
        return AsyncSummaryResourceWithRawResponse(self._security.summary)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResourceWithRawResponse:
        return AsyncTimeseriesGroupsResourceWithRawResponse(self._security.timeseries_groups)


class SecurityResourceWithStreamingResponse:
    def __init__(self, security: SecurityResource) -> None:
        self._security = security

        self.summary_v2 = to_streamed_response_wrapper(
            security.summary_v2,
        )
        self.timeseries_groups_v2 = to_streamed_response_wrapper(
            security.timeseries_groups_v2,
        )

    @cached_property
    def top(self) -> TopResourceWithStreamingResponse:
        return TopResourceWithStreamingResponse(self._security.top)

    @cached_property
    def summary(self) -> SummaryResourceWithStreamingResponse:
        return SummaryResourceWithStreamingResponse(self._security.summary)

    @cached_property
    def timeseries_groups(self) -> TimeseriesGroupsResourceWithStreamingResponse:
        return TimeseriesGroupsResourceWithStreamingResponse(self._security.timeseries_groups)


class AsyncSecurityResourceWithStreamingResponse:
    def __init__(self, security: AsyncSecurityResource) -> None:
        self._security = security

        self.summary_v2 = async_to_streamed_response_wrapper(
            security.summary_v2,
        )
        self.timeseries_groups_v2 = async_to_streamed_response_wrapper(
            security.timeseries_groups_v2,
        )

    @cached_property
    def top(self) -> AsyncTopResourceWithStreamingResponse:
        return AsyncTopResourceWithStreamingResponse(self._security.top)

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithStreamingResponse:
        return AsyncSummaryResourceWithStreamingResponse(self._security.summary)

    @cached_property
    def timeseries_groups(self) -> AsyncTimeseriesGroupsResourceWithStreamingResponse:
        return AsyncTimeseriesGroupsResourceWithStreamingResponse(self._security.timeseries_groups)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DynamicRoutingListVersionsResponse", "Data", "DataVersion"]


class DataVersion(BaseModel):
    active: Literal["true", "false"]

    created_at: str

    data: str

    version_id: str

    is_valid: Optional[bool] = None


class Data(BaseModel):
    order_by: str

    order_by_direction: str

    page: float

    per_page: float

    versions: List[DataVersion]


class DynamicRoutingListVersionsResponse(BaseModel):
    data: Data

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "EmailSendingSendParams",
    "From",
    "FromEmailSendingEmailAddressObject",
    "Attachment",
    "AttachmentEmailSendingEmailInlineAttachment",
    "AttachmentEmailSendingEmailAttachment",
    "ReplyTo",
    "ReplyToEmailSendingEmailAddressObject",
]


class EmailSendingSendParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of the account."""

    from_: Required[Annotated[From, PropertyInfo(alias="from")]]
    """Sender email address. Either a plain string or an object with address and name."""

    subject: Required[str]
    """Email subject line."""

    to: Required[Union[str, SequenceNotStr[str]]]
    """Recipient(s). A single email string or an array of email strings."""

    attachments: Iterable[Attachment]
    """File attachments and inline images."""

    bcc: Union[str, SequenceNotStr[str]]
    """BCC recipient(s). A single email string or an array of email strings."""

    cc: Union[str, SequenceNotStr[str]]
    """CC recipient(s). A single email string or an array of email strings."""

    headers: Dict[str, str]
    """Custom email headers as key-value pairs."""

    html: str
    """HTML body of the email. At least one of text or html must be provided."""

    reply_to: ReplyTo
    """Reply-to address. Either a plain string or an object with address and name."""

    text: str
    """Plain text body of the email. At least one of text or html must be provided."""


class FromEmailSendingEmailAddressObject(TypedDict, total=False):
    address: Required[str]
    """Email address (e.g., 'user@example.com')."""

    name: Required[str]
    """Display name for the email address (e.g., 'John Doe')."""


From: TypeAlias = Union[str, FromEmailSendingEmailAddressObject]


class AttachmentEmailSendingEmailInlineAttachment(TypedDict, total=False):
    content: Required[str]
    """Base64-encoded content of the attachment."""

    content_id: Required[str]
    """
    Content ID used to reference this attachment in HTML via cid: URI (e.g.,
    <img src="cid:logo">).
    """

    disposition: Required[Literal["inline"]]
    """Must be 'inline'. Indicates the attachment is embedded in the email body."""

    filename: Required[str]
    """Filename for the attachment."""

    type: Required[str]
    """MIME type of the attachment (e.g., 'image/png', 'text/plain')."""


class AttachmentEmailSendingEmailAttachment(TypedDict, total=False):
    content: Required[str]
    """Base64-encoded content of the attachment."""

    disposition: Required[Literal["attachment"]]
    """Must be 'attachment'. Indicates a standard file attachment."""

    filename: Required[str]
    """Filename for the attachment."""

    type: Required[str]
    """MIME type of the attachment (e.g., 'application/pdf', 'text/plain')."""


Attachment: TypeAlias = Union[AttachmentEmailSendingEmailInlineAttachment, AttachmentEmailSendingEmailAttachment]


class ReplyToEmailSendingEmailAddressObject(TypedDict, total=False):
    address: Required[str]
    """Email address (e.g., 'user@example.com')."""

    name: Required[str]
    """Display name for the email address (e.g., 'John Doe')."""


ReplyTo: TypeAlias = Union[str, ReplyToEmailSendingEmailAddressObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EndpointHealthcheckUpdateParams"]


class EndpointHealthcheckUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    check_type: Required[Literal["icmp"]]
    """type of check to perform"""

    endpoint: Required[str]
    """the IP address of the host to perform checks against"""

    name: str
    """Optional name associated with this check"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options

__all__ = ["PageResource", "AsyncPageResource"]


class PageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PageResourceWithStreamingResponse(self)

    def get(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Establishes a WebSocket connection to a specific Chrome DevTools target or page.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID, e.g. page ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/page/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPageResourceWithStreamingResponse(self)

    async def get(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Establishes a WebSocket connection to a specific Chrome DevTools target or page.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID, e.g. page ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/page/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PageResourceWithRawResponse:
    def __init__(self, page: PageResource) -> None:
        self._page = page

        self.get = to_raw_response_wrapper(
            page.get,
        )


class AsyncPageResourceWithRawResponse:
    def __init__(self, page: AsyncPageResource) -> None:
        self._page = page

        self.get = async_to_raw_response_wrapper(
            page.get,
        )


class PageResourceWithStreamingResponse:
    def __init__(self, page: PageResource) -> None:
        self._page = page

        self.get = to_streamed_response_wrapper(
            page.get,
        )


class AsyncPageResourceWithStreamingResponse:
    def __init__(self, page: AsyncPageResource) -> None:
        self._page = page

        self.get = async_to_streamed_response_wrapper(
            page.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["DNSParam"]


class DNSParam(TypedDict, total=False):
    """The name and type of DNS record for the Spectrum application."""

    name: str
    """The name of the DNS record associated with the application."""

    type: Literal["CNAME", "ADDRESS"]
    """The type of DNS record associated with the application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .aspa import (
    ASPAResource,
    AsyncASPAResource,
    ASPAResourceWithRawResponse,
    AsyncASPAResourceWithRawResponse,
    ASPAResourceWithStreamingResponse,
    AsyncASPAResourceWithStreamingResponse,
)
from .roas import (
    RoasResource,
    AsyncRoasResource,
    RoasResourceWithRawResponse,
    AsyncRoasResourceWithRawResponse,
    RoasResourceWithStreamingResponse,
    AsyncRoasResourceWithStreamingResponse,
)
from .rpki import (
    RPKIResource,
    AsyncRPKIResource,
    RPKIResourceWithRawResponse,
    AsyncRPKIResourceWithRawResponse,
    RPKIResourceWithStreamingResponse,
    AsyncRPKIResourceWithStreamingResponse,
)

__all__ = [
    "ASPAResource",
    "AsyncASPAResource",
    "ASPAResourceWithRawResponse",
    "AsyncASPAResourceWithRawResponse",
    "ASPAResourceWithStreamingResponse",
    "AsyncASPAResourceWithStreamingResponse",
    "RoasResource",
    "AsyncRoasResource",
    "RoasResourceWithRawResponse",
    "AsyncRoasResourceWithRawResponse",
    "RoasResourceWithStreamingResponse",
    "AsyncRoasResourceWithStreamingResponse",
    "RPKIResource",
    "AsyncRPKIResource",
    "RPKIResourceWithRawResponse",
    "AsyncRPKIResourceWithRawResponse",
    "RPKIResourceWithStreamingResponse",
    "AsyncRPKIResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ips import (
    IPsResource,
    AsyncIPsResource,
    IPsResourceWithRawResponse,
    AsyncIPsResourceWithRawResponse,
    IPsResourceWithStreamingResponse,
    AsyncIPsResourceWithStreamingResponse,
)

__all__ = [
    "IPsResource",
    "AsyncIPsResource",
    "IPsResourceWithRawResponse",
    "AsyncIPsResourceWithRawResponse",
    "IPsResourceWithStreamingResponse",
    "AsyncIPsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SkipConfigurationParam"]


class SkipConfigurationParam(TypedDict, total=False):
    """Content types to exclude from context analysis and return all matches."""

    files: Required[bool]
    """If the content type is a file, skip context analysis and return all matches."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["SecretDuplicateParams"]


class SecretDuplicateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account Identifier"""

    store_id: Required[str]
    """Store Identifier"""

    name: Required[str]
    """The name of the secret"""

    scopes: Required[SequenceNotStr[str]]
    """The list of services that can use this secret."""

    comment: str
    """Freeform text describing the secret"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .sso import (
    SSOResource,
    AsyncSSOResource,
    SSOResourceWithRawResponse,
    AsyncSSOResourceWithRawResponse,
    SSOResourceWithStreamingResponse,
    AsyncSSOResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .resource_groups import (
    ResourceGroupsResource,
    AsyncResourceGroupsResource,
    ResourceGroupsResourceWithRawResponse,
    AsyncResourceGroupsResourceWithRawResponse,
    ResourceGroupsResourceWithStreamingResponse,
    AsyncResourceGroupsResourceWithStreamingResponse,
)
from .permission_groups import (
    PermissionGroupsResource,
    AsyncPermissionGroupsResource,
    PermissionGroupsResourceWithRawResponse,
    AsyncPermissionGroupsResourceWithRawResponse,
    PermissionGroupsResourceWithStreamingResponse,
    AsyncPermissionGroupsResourceWithStreamingResponse,
)
from .user_groups.user_groups import (
    UserGroupsResource,
    AsyncUserGroupsResource,
    UserGroupsResourceWithRawResponse,
    AsyncUserGroupsResourceWithRawResponse,
    UserGroupsResourceWithStreamingResponse,
    AsyncUserGroupsResourceWithStreamingResponse,
)

__all__ = ["IAMResource", "AsyncIAMResource"]


class IAMResource(SyncAPIResource):
    @cached_property
    def permission_groups(self) -> PermissionGroupsResource:
        return PermissionGroupsResource(self._client)

    @cached_property
    def resource_groups(self) -> ResourceGroupsResource:
        return ResourceGroupsResource(self._client)

    @cached_property
    def user_groups(self) -> UserGroupsResource:
        return UserGroupsResource(self._client)

    @cached_property
    def sso(self) -> SSOResource:
        return SSOResource(self._client)

    @cached_property
    def with_raw_response(self) -> IAMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IAMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IAMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IAMResourceWithStreamingResponse(self)


class AsyncIAMResource(AsyncAPIResource):
    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResource:
        return AsyncPermissionGroupsResource(self._client)

    @cached_property
    def resource_groups(self) -> AsyncResourceGroupsResource:
        return AsyncResourceGroupsResource(self._client)

    @cached_property
    def user_groups(self) -> AsyncUserGroupsResource:
        return AsyncUserGroupsResource(self._client)

    @cached_property
    def sso(self) -> AsyncSSOResource:
        return AsyncSSOResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIAMResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIAMResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIAMResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIAMResourceWithStreamingResponse(self)


class IAMResourceWithRawResponse:
    def __init__(self, iam: IAMResource) -> None:
        self._iam = iam

    @cached_property
    def permission_groups(self) -> PermissionGroupsResourceWithRawResponse:
        return PermissionGroupsResourceWithRawResponse(self._iam.permission_groups)

    @cached_property
    def resource_groups(self) -> ResourceGroupsResourceWithRawResponse:
        return ResourceGroupsResourceWithRawResponse(self._iam.resource_groups)

    @cached_property
    def user_groups(self) -> UserGroupsResourceWithRawResponse:
        return UserGroupsResourceWithRawResponse(self._iam.user_groups)

    @cached_property
    def sso(self) -> SSOResourceWithRawResponse:
        return SSOResourceWithRawResponse(self._iam.sso)


class AsyncIAMResourceWithRawResponse:
    def __init__(self, iam: AsyncIAMResource) -> None:
        self._iam = iam

    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResourceWithRawResponse:
        return AsyncPermissionGroupsResourceWithRawResponse(self._iam.permission_groups)

    @cached_property
    def resource_groups(self) -> AsyncResourceGroupsResourceWithRawResponse:
        return AsyncResourceGroupsResourceWithRawResponse(self._iam.resource_groups)

    @cached_property
    def user_groups(self) -> AsyncUserGroupsResourceWithRawResponse:
        return AsyncUserGroupsResourceWithRawResponse(self._iam.user_groups)

    @cached_property
    def sso(self) -> AsyncSSOResourceWithRawResponse:
        return AsyncSSOResourceWithRawResponse(self._iam.sso)


class IAMResourceWithStreamingResponse:
    def __init__(self, iam: IAMResource) -> None:
        self._iam = iam

    @cached_property
    def permission_groups(self) -> PermissionGroupsResourceWithStreamingResponse:
        return PermissionGroupsResourceWithStreamingResponse(self._iam.permission_groups)

    @cached_property
    def resource_groups(self) -> ResourceGroupsResourceWithStreamingResponse:
        return ResourceGroupsResourceWithStreamingResponse(self._iam.resource_groups)

    @cached_property
    def user_groups(self) -> UserGroupsResourceWithStreamingResponse:
        return UserGroupsResourceWithStreamingResponse(self._iam.user_groups)

    @cached_property
    def sso(self) -> SSOResourceWithStreamingResponse:
        return SSOResourceWithStreamingResponse(self._iam.sso)


class AsyncIAMResourceWithStreamingResponse:
    def __init__(self, iam: AsyncIAMResource) -> None:
        self._iam = iam

    @cached_property
    def permission_groups(self) -> AsyncPermissionGroupsResourceWithStreamingResponse:
        return AsyncPermissionGroupsResourceWithStreamingResponse(self._iam.permission_groups)

    @cached_property
    def resource_groups(self) -> AsyncResourceGroupsResourceWithStreamingResponse:
        return AsyncResourceGroupsResourceWithStreamingResponse(self._iam.resource_groups)

    @cached_property
    def user_groups(self) -> AsyncUserGroupsResourceWithStreamingResponse:
        return AsyncUserGroupsResourceWithStreamingResponse(self._iam.user_groups)

    @cached_property
    def sso(self) -> AsyncSSOResourceWithStreamingResponse:
        return AsyncSSOResourceWithStreamingResponse(self._iam.sso)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ObjectUploadParams"]


class ObjectUploadParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    bucket_name: Required[str]
    """Name of the bucket."""

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""

    cf_r2_storage_class: Annotated[Literal["Standard", "InfrequentAccess"], PropertyInfo(alias="cf-r2-storage-class")]
    """Storage class for newly uploaded objects, unless specified otherwise."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["CatchAllAction"]


class CatchAllAction(BaseModel):
    """Action for the catch-all routing rule."""

    type: Literal["drop", "forward", "worker"]
    """Type of action for catch-all rule."""

    value: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Namespace"]


class Namespace(BaseModel):
    id: Optional[str] = None

    class_: Optional[str] = FieldInfo(alias="class", default=None)

    name: Optional[str] = None

    script: Optional[str] = None

    use_sqlite: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.token_validation.configuration import credential_update_params
from ....types.token_validation.configuration.credential_update_response import CredentialUpdateResponse

__all__ = ["CredentialsResource", "AsyncCredentialsResource"]


class CredentialsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CredentialsResourceWithStreamingResponse(self)

    def update(
        self,
        config_id: str,
        *,
        zone_id: str,
        keys: Iterable[credential_update_params.Key],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialUpdateResponse:
        """
        Update Token Configuration credentials

        Args:
          zone_id: Identifier.

          config_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/token_validation/config/{config_id}/credentials", zone_id=zone_id, config_id=config_id
            ),
            body=maybe_transform({"keys": keys}, credential_update_params.CredentialUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialUpdateResponse,
        )


class AsyncCredentialsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCredentialsResourceWithStreamingResponse(self)

    async def update(
        self,
        config_id: str,
        *,
        zone_id: str,
        keys: Iterable[credential_update_params.Key],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CredentialUpdateResponse:
        """
        Update Token Configuration credentials

        Args:
          zone_id: Identifier.

          config_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/token_validation/config/{config_id}/credentials", zone_id=zone_id, config_id=config_id
            ),
            body=await async_maybe_transform({"keys": keys}, credential_update_params.CredentialUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CredentialUpdateResponse,
        )


class CredentialsResourceWithRawResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.update = to_raw_response_wrapper(
            credentials.update,
        )


class AsyncCredentialsResourceWithRawResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.update = async_to_raw_response_wrapper(
            credentials.update,
        )


class CredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.update = to_streamed_response_wrapper(
            credentials.update,
        )


class AsyncCredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.update = async_to_streamed_response_wrapper(
            credentials.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.bgp.ips import top_ases_params
from .....types.radar.bgp.ips.top_ases_response import TopAsesResponse

__all__ = ["TopResource", "AsyncTopResource"]


class TopResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TopResourceWithStreamingResponse(self)

    def ases(
        self,
        *,
        country: str | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        metric: Literal["v4_24s", "v6_48s"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopAsesResponse:
        """
        Returns the top-N autonomous systems by announced IP space at the nearest 8-hour
        RIB boundary at or before the requested date. The snapped boundary is returned
        as `anchor_ts`.

        Args:
          country: Optional ISO 3166-1 alpha-2 country filter. Omit for global top-N.

          date: Filters results by the specified datetime (ISO 8601).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          metric: Ranking metric: IPv4 /24 count or IPv6 /48 count.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/ips/top/ases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country": country,
                        "date": date,
                        "format": format,
                        "limit": limit,
                        "metric": metric,
                    },
                    top_ases_params.TopAsesParams,
                ),
                post_parser=ResultWrapper[TopAsesResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopAsesResponse], ResultWrapper[TopAsesResponse]),
        )


class AsyncTopResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTopResourceWithStreamingResponse(self)

    async def ases(
        self,
        *,
        country: str | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        metric: Literal["v4_24s", "v6_48s"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopAsesResponse:
        """
        Returns the top-N autonomous systems by announced IP space at the nearest 8-hour
        RIB boundary at or before the requested date. The snapped boundary is returned
        as `anchor_ts`.

        Args:
          country: Optional ISO 3166-1 alpha-2 country filter. Omit for global top-N.

          date: Filters results by the specified datetime (ISO 8601).

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          metric: Ranking metric: IPv4 /24 count or IPv6 /48 count.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/ips/top/ases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country": country,
                        "date": date,
                        "format": format,
                        "limit": limit,
                        "metric": metric,
                    },
                    top_ases_params.TopAsesParams,
                ),
                post_parser=ResultWrapper[TopAsesResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopAsesResponse], ResultWrapper[TopAsesResponse]),
        )


class TopResourceWithRawResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.ases = to_raw_response_wrapper(
            top.ases,
        )


class AsyncTopResourceWithRawResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.ases = async_to_raw_response_wrapper(
            top.ases,
        )


class TopResourceWithStreamingResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.ases = to_streamed_response_wrapper(
            top.ases,
        )


class AsyncTopResourceWithStreamingResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.ases = async_to_streamed_response_wrapper(
            top.ases,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ......_types import SequenceNotStr

__all__ = ["TagUpdateParams"]


class TagUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    dispatch_namespace: Required[str]
    """Name of the Workers for Platforms dispatch namespace."""

    body: Required[Optional[SequenceNotStr[str]]]
    """Tags associated with the Worker."""
